/*
 * File event notification support.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <condition_variable>
#include <list>
#include <mutex>

#include <fsclient/file.h>
#include <ipc/server.h>
#include <systypes/base.h>



/* An object for monitoring file event notifications. */

class FileNotifier
{
protected:
    std::list<file_t *> _affected;
    l4_cap_idx_t _thread = L4_INVALID_CAP;
    bool _started = false;

    std::mutex _lock;
    std::condition_variable _notified;

    /* Helper methods. */

    virtual void _notify(file_t *file, notify_flags_t flags);

public:
    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<FileNotifier *>(this); }

    /* Local operations. */

    virtual long subscribe(file_t *file, notify_flags_t flags);

    virtual long unsubscribe(file_t *file);

    virtual void mainloop();

    virtual long start();

    virtual long wait(file_t **file);
};



/* Helper functions. */

FileNotifier *get_notifier();

// vim: tabstop=4 expandtab shiftwidth=4
